# Generated by Django 5.0 on 2025-09-17 12:15

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('services_communication', '0005_publishedeventqueue_rename_chanel_20250313_0733'),
    ]

    operations = [
        migrations.AddField(
            model_name='publishedeventqueue',
            name='properties',
            field=models.JSONField(null=True),
        ),
    ]
