# Generated by Django 4.2.2 on 2023-06-14 16:39

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='PublishedEventQueue',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('exchange', models.CharField(max_length=64)),
                ('routing_key', models.CharField(max_length=256)),
                ('event_time', models.DateTimeField(auto_now_add=True)),
                ('aggregate', models.CharField(max_length=64)),
                ('event_type', models.CharField(max_length=64)),
                ('payload', models.JSONField()),
            ],
        ),
    ]
