# Generated by Django 4.2.17 on 2024-12-18 10:31

import django.core.serializers.json
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('services_communication', '0002_publishedeventqueue_create_time_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='publishedeventqueue',
            name='body',
            field=models.JSONField(encoder=django.core.serializers.json.DjangoJSONEncoder, null=True),
        ),
        migrations.AddField(
            model_name='publishedeventqueue',
            name='is_new_style',
            field=models.BooleanField(null=True),
        ),
        migrations.AddField(
            model_name='publishedeventqueue',
            name='send_after_time',
            field=models.DateTimeField(null=True),
        ),
        migrations.AlterField(
            model_name='publishedeventqueue',
            name='aggregate',
            field=models.CharField(max_length=64, null=True),
        ),
        migrations.AlterField(
            model_name='publishedeventqueue',
            name='event_time',
            field=models.DateTimeField(null=True),
        ),
        migrations.AlterField(
            model_name='publishedeventqueue',
            name='event_type',
            field=models.CharField(max_length=64, null=True),
        ),
        migrations.AlterField(
            model_name='publishedeventqueue',
            name='payload',
            field=models.JSONField(encoder=django.core.serializers.json.DjangoJSONEncoder, null=True),
        ),
        migrations.SeparateDatabaseAndState(
            state_operations=[
                migrations.AlterField(
                    model_name='publishedeventqueue',
                    name='id',
                    field=models.IntegerField(primary_key=True, serialize=False),
                ),
            ],
        )
    ]
