# Generated by Django 4.2.2 on 2024-03-13 12:32

import django.core.serializers.json
from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('services_communication', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='publishedeventqueue',
            name='create_time',
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='publishedeventqueue',
            name='tags',
            field=models.JSONField(null=True),
        ),
        migrations.AlterField(
            model_name='publishedeventqueue',
            name='event_time',
            field=models.DateTimeField(),
        ),
        migrations.AlterField(
            model_name='publishedeventqueue',
            name='payload',
            field=models.JSONField(encoder=django.core.serializers.json.DjangoJSONEncoder),
        ),
    ]
